"""
Module to include the functionality related to the
parameter values and how we define them to be used
in our 'yta-editor' video editor library.
"""
from yta_editor_parameters.values.constant import ConstantVideoEditorParameterValue
from yta_editor_parameters.values.eased import EasedVideoEditorParameterValue


__all__ = [
    'ConstantVideoEditorParameterValue',
    'EasedVideoEditorParameterValue'
]


from yta_editor_parameters.values.abstract import VideoEditorParameterValue
from yta_editor_parameters.values import ConstantVideoEditorParameterValue
from yta_validation import PythonValidator


def parse_parameters(
    parameters: dict
) -> dict[str, 'VideoEditorParameterValue']:
    """
    Parse the `parameters` provided and raise an exception
    if some of them is unaccepted. It will transform the
    basic types into `ConstantVideoEditorParameterValue`
    instances, and return the new dict with the values
    transformed.

    These types will be transformed into constant values:
    - `int`
    - `float`
    - `bool`
    - `str`
    - `None`
    """
    parameters_parsed = {}

    for name, value in parameters.items():
        try:
            parameters_parsed[name] = parse_parameter(value)
        except:
            raise Exception(f'The "{name}" parameter is not a "VideoEditorParameterValue" nor a basic and non iterable type') 

    return parameters_parsed

def parse_parameter(
    parameter: VideoEditorParameterValue
) -> VideoEditorParameterValue:
    """
    Parse the `parameter` provided and raise an exception
    if it is not accepted. It will transform the basic
    types into `ConstantVideoEditorParameterValue` instances,
    and return the parameter but casted.

    These types will be transformed into constant values:
    - `int`
    - `float`
    - `bool`
    - `str`
    - `None`
    """
    if PythonValidator.is_instance_of(parameter, VideoEditorParameterValue):
        return parameter

    if (
        PythonValidator.is_basic_non_iterable_type(parameter) or
        parameter is None
    ):
        return ConstantVideoEditorParameterValue(parameter)
        
    raise Exception(f'The parameter is not a "VideoEditorParameterValue" nor a basic and non iterable type')

def is_this_constant_value(
    parameter: VideoEditorParameterValue,
    value: 'BASIC_NON_ITERABLE_TYPE'
) -> bool:
    """
    Check if the `parameter` provided is a 
    `ConstantVideoEditorParameterValue` and has the
    `value` provided.

    This is useful to check if a parameter is
    the default value and we don't need to apply
    any change because of this.
    """
    # TODO: We could add an Eased but with the
    # same 'start_value' and 'end_value' because
    # the evaluation will be the same always
    return (
        PythonValidator.is_instance_of(parameter, ConstantVideoEditorParameterValue) and
        parameter.value == value
    )


from yta_editor_parameters.values.abstract import VideoEditorParameterValue
from yta_editor_time.evaluation_context import EvaluationContext


class ConstantVideoEditorParameterValue(VideoEditorParameterValue):
    """
    A parameter value that has always the same value
    and is unaware of the context.
    """

    def __init__(
        self,
        value: 'BASIC_NON_ITERABLE_TYPE'
    ):
        self.value: 'BASIC_NON_ITERABLE_TYPE' = value
        """
        The constant value of this parameter.
        """

    def evaluate(
        self,
        # This is mandatory, even though it is not used
        evaluation_context: EvaluationContext
    ):
        return self.value


from yta_editor_time.evaluation_context import EvaluationContext
from abc import abstractmethod, ABC


class VideoEditorParameterValue(ABC):
    """
    A parameter value that uses a video editor
    context to be evaluated.
    """

    @abstractmethod
    def evaluate(
        self,
        # This is mandatory, even though it is not used
        evaluation_context: EvaluationContext
    ):
        pass


from typing import Union


BASIC_NON_ITERABLE_TYPE = Union[int, float, bool, str]
"""
The basic non iterable type we can accept as
a parameter and that will be transformed into
a `ConstantValue`, including:
- `int`
- `float`
- `bool`
- `str`
"""